/****************************************************************************
** templatewizard.cpp file.
**
** This file implements the TemplateWizardDialog.
**
** The following structure is assumed:
** /usr/share/qdvdauthor/template/static/<TemplateName>/<Files>
** /usr/share/qdvdauthor/template/animated/<TemplateName>/<Files>
** /usr/share/qdvdauthor/template/transition/<TemplateName>/<Files>
**
** Here are the files required :
** template.xml -> holds information about the template, such as comment
**                 NrOfDropZones, coordinates of DropZones etc.
** thumbnail.png-> Image to display in selectionBox ( width=200 )
** background/backgroundXXXXX.jpg where XXXXX = sequenceNo. 
** background/background00000.jpg REQUIRED.
** template.mpg -> preview for menus
** dropZone[X].mpg -> default for the dropZone, with X = [1 .. N]
** 
** The requirements are simple: 
** Input, a few user mouse clicks
** Output, one or multiple dvd menu(s)
** 
** Interface to the program : QValueList<DVDMenu *>
** 
*****************************************************************************/

#include <qdir.h>
#include <qtimer.h>
#include <qcursor.h>
#include <qpainter.h>
#include <qfileinfo.h>
#include <qiconview.h>
#include <qcombobox.h>
#include <qfiledialog.h>
#include <qmessagebox.h>

// Include the generated config files ...
#include "CONFIG.h"
#include "xml_dvd.h"
#include "utils.h"
#include "qdvdauthor.h"
#include "dvdmenu.h"
#include "qplayer/mediainfo.h"
#include "qplayer/mediacreator.h"

#include "templatewizard.h"

/****************************************************************************
** TemplateFrame - class
**
** Simply there to emit a signal when some one clicks in it.
**
*****************************************************************************/
TemplateFrame::TemplateFrame ( QWidget *pParent, const char *pName, WFlags flags )
	: QFrame (pParent, pName, flags)
{
}

TemplateFrame::~TemplateFrame ()
{
}

void TemplateFrame::mouseReleaseEvent ( QMouseEvent *pEvent )
{
	emit (signalMouseReleaseEvent (pEvent));
}

TemplateWizard::DropZone::DropZone (int i, QString s, QRect r)
{
	bMovie = false;
	iNumber=i; 
	qsText=s; 
	rect=r;
	pPixmap = NULL;
}

TemplateWizard::DropZone::~DropZone ()
{
	if (pPixmap)
		delete pPixmap;
}



TemplateWizard::TemplateWizard( QWidget* parent, const char* name, WFlags fl )
    : uiTemplateWizard( parent, name, fl )
{
	m_pDVDAuthor = NULL;
	m_pDVDMenu   = NULL;
	m_fScale     = 0.0f;
	m_bDontRestart = false;
}

TemplateWizard::~TemplateWizard()
{
	uint t;
	m_bDontRestart = true;
	for (t=0;t<m_listDropZones.count();t++)
		delete m_listDropZones[t];
	if (m_pMediaInterface)	{
		m_pMediaInterface->stop();
		delete m_pMediaInterface;
	}
}

void TemplateWizard::hide ()
{
	m_bDontRestart = true;
	if (m_pMediaInterface)
		m_pMediaInterface->stop();

	uiTemplateWizard::hide();
}

void TemplateWizard::initMe (QDVDAuthor*pDVDAuthor)
{
	m_pDVDAuthor = pDVDAuthor;
	refreshComboMenu ();
	setNextEnabled   ( page (0), false );
	setFinishEnabled ( page (1), true  );

	initTab (QString ("static"), m_pIconViewStatic);
	initTab (QString ("animated"), m_pIconViewAnimated);
	initTab (QString ("transition"), m_pIconViewTransition);

	QGridLayout *WizardPageLayout_2 = new QGridLayout( WizardPage_2, 1, 1, 11, 6, "WizardPageLayout_2"); 
	QSpacerItem *spacer1 = new QSpacerItem( 20, 2, QSizePolicy::Minimum, QSizePolicy::Expanding );
	WizardPageLayout_2->addItem( spacer1, 0, 1 );

	m_pFrameProperties = new TemplateFrame( WizardPage_2, "m_pFrameProperties" );
	m_pFrameProperties->setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)1, (QSizePolicy::SizeType)1, 2, 2, m_pFrameProperties->sizePolicy().hasHeightForWidth() ) );
	m_pFrameProperties->setFrameShape( QFrame::StyledPanel );
	m_pFrameProperties->setFrameShadow( QFrame::Raised );
	connect (m_pFrameProperties, SIGNAL(signalMouseReleaseEvent (QMouseEvent *)), this, SLOT(slotTemplateClicked (QMouseEvent *)));

	WizardPageLayout_2->addWidget( m_pFrameProperties, 1, 1 );
	QSpacerItem *spacer1_2 = new QSpacerItem( 20, 2, QSizePolicy::Minimum, QSizePolicy::Expanding );
	WizardPageLayout_2->addItem( spacer1_2, 2, 1 );
	QSpacerItem *spacer3 = new QSpacerItem( 2, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	WizardPageLayout_2->addItem( spacer3, 1, 2 );
	QSpacerItem *spacer3_2 = new QSpacerItem( 2, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
	WizardPageLayout_2->addItem( spacer3_2, 1, 0 );
    
	connect (m_pComboMenu, SIGNAL(activated (const QString &)), this, SLOT(slotTargetMenuChanged (const QString &)));
	resize( QSize(739, 536).expandedTo(minimumSizeHint()) );
	clearWState( WState_Polished );
	initPreview ();
}

void TemplateWizard::slotPlaybackFinished ()
{
//printf ("TemplateWizard::slotPlaybackFinished <%d>\n", m_bDontRestart);
	// TODO: check elapsed time and break this one here if < 2 seconds.
	if ( ! m_bDontRestart )
		m_pMediaInterface->play ();
	m_bDontRestart = false;
}

void TemplateWizard::refreshComboMenu ()
{
	uint t;
	m_pComboMenu->clear ();

	if (m_qsCurrentTemplate.isEmpty())
		m_qsCurrentTemplate = QString ("New Menu");

	if (m_pDVDAuthor)	{
		// Here we ensure we have a unique name
		m_qsCurrentTemplate = m_pDVDAuthor->getUniqueMenuTitle (m_qsCurrentTemplate, 0);
	}
	m_pComboMenu->insertItem (m_qsCurrentTemplate);
	m_pComboMenu->insertItem (VMGM_LABEL);
	if ( ! m_pDVDAuthor )
		return;
	QValueList<DVDMenu *> listMenus = m_pDVDAuthor->getSubMenus ();
	for (t=0;t<listMenus.count();t++)	{
		m_pComboMenu->insertItem (listMenus[t]->name());
	}
}

void TemplateWizard::initPreview ()
{
	m_pLabelPreview->hide();

	// Here we create the mediaEngine for this dialog.
	MediaInfo *pMediaInfo   = m_pDVDAuthor->getMediaInfo  ();
	void      *pMediaEngine = pMediaInfo->getMediaEngine ();
	m_pMediaInterface = MediaCreator::createPreferredWidget (m_pFramePreview, "TemplateMediaWidget", pMediaEngine);
	connect (m_pMediaInterface, SIGNAL(signalPlaybackFinished()), this, SLOT(slotPlaybackFinished()));
	
	QGridLayout *pVideoLayout = new QGridLayout (m_pFramePreview, 1, 1, 2, 2, "MediaMainLayout");
	pVideoLayout->addWidget (m_pMediaInterface->getWidget(), 0, 0);
	// This is taken care of in the polish () function for Xine and not necessary for MPlayer ...
	m_pMediaInterface->initMediaEngine();
}

void TemplateWizard::initTab (QString qsWhichTab, QIconView *pIconView)
{
	uint t;
	QStringList listTemplates;
	QIconViewItem *pItem;
	QString qsPrefixDir, qsPixmap, qsXMLFile, qsBackgroundFile;
	QFileInfo fileInfo;
	QPixmap thePixmap;
	QDir theTemplates; 
	QSize grid (50, 50);
	qsPrefixDir = QString ("%1/share/qdvdauthor/%2").arg(PREFIX_DIRECTORY).arg(qsWhichTab);

	if ( ! theTemplates.exists (qsPrefixDir) )	{
		pItem = new QIconViewItem (pIconView, tr ("Warning no dir :\n%1").arg (qsPrefixDir));
		return;
	}

	pIconView->arrangeItemsInGrid (grid, TRUE);
	
	theTemplates.cd   (qsPrefixDir, TRUE);
	listTemplates = theTemplates.entryList (QDir::Dirs, QDir::Name);
	if (listTemplates.count () < 3)	{
		pItem = new QIconViewItem (pIconView, tr ("No %1 Backgrounds in dir :\n%2").arg(qsWhichTab).arg(qsPrefixDir));
		return;
	}
	for (t=0;t<listTemplates.count ();t++)	{
		// Here we go through all directories and check if the preview is present.
		// if so, then we snap the preview.jpg and create a ListItem
		if ( (listTemplates[t] == ".") || (listTemplates[t] == ".") )
			continue;
		qsPixmap = QString ("%1/%2/%3").arg(qsPrefixDir).arg(listTemplates[t]).arg("thumbnail.png");
		fileInfo.setFile (qsPixmap);
		if ( ! fileInfo.exists() )
			continue;
		qsXMLFile = QString ("%1/%2/%3").arg(qsPrefixDir).arg(listTemplates[t]).arg("template.xml");
		fileInfo.setFile (qsXMLFile);
		if ( ! fileInfo.exists() )
			continue;
		qsBackgroundFile = QString ("%1/%2/background/background00000.jpg").arg(qsPrefixDir).arg(listTemplates[t]);
		fileInfo.setFile (qsBackgroundFile);
		if ( ! fileInfo.exists() )
			continue;
		thePixmap = QPixmap (qsPixmap);
		pItem = new QIconViewItem (pIconView, listTemplates[t], thePixmap);
	}
	connect (pIconView, SIGNAL(clicked (QIconViewItem *)), this, SLOT(slotIconViewClicked (QIconViewItem *)));
}

void TemplateWizard::slotIconViewClicked (QIconViewItem *pIconViewItem)
{
	// The user clicked one of the Icons, 
	// so we should get theinformation, and read in the description from template.xml
	if ( ! pIconViewItem )
		return;
	QString qsDescription, qsFileName;
	QPixmap thumbnail;

	// First we determine where the click came from
	m_bDontRestart = true;
	m_pMediaInterface->stop ();
	if (m_pIconViewStatic->index (pIconViewItem) != -1)	{	// Display image
		m_qsPathName = QString ("%1/share/qdvdauthor/static/%2/").arg(PREFIX_DIRECTORY).arg(pIconViewItem->text());
		thumbnail = QPixmap(m_qsPathName+QString ("thumbnail.png"));
		m_pLabelPreview->show ();
		m_pFramePreview->hide ();
		m_pLabelPreview->setPixmap    (thumbnail);
		m_pLabelPreview->setFixedSize (thumbnail.size());
	}
	else 	{	// Play movie ...
		if (m_pIconViewAnimated->index (pIconViewItem) != -1)
			m_qsPathName = QString ("%1/share/qdvdauthor/animated/%2/").arg(PREFIX_DIRECTORY).arg(pIconViewItem->text());
		else if (m_pIconViewTransition->index (pIconViewItem) != -1)
			m_qsPathName = QString ("%1/share/qdvdauthor/transition/%2/").arg(PREFIX_DIRECTORY).arg(pIconViewItem->text());
		else
			return;
		m_pLabelPreview->hide();
		m_pFramePreview->show();
		thumbnail = QPixmap(m_qsPathName+QString ("thumbnail.png"));
		m_pFramePreview->setFixedSize (thumbnail.size());
		m_pMediaInterface->playMRL (m_qsPathName+QString ("thumbnail.mpg"));
		m_bDontRestart = false;
	}
	qsFileName = m_qsPathName + QString ("template.xml");
	qsDescription = readProjectFile (qsFileName, true);
	m_pLabelTemplateDescription->setText (qsDescription);
	refreshComboMenu ();
	m_pComboMenu->setEditable ( true );
	setNextEnabled ( page (0), true );
}

void TemplateWizard::slotTargetMenuChanged (const QString &qsMenuName)
{
	// Here we check if we have a new or an existing menu.
	bool bEditable = true;

	if ( m_pDVDAuthor )	{
		if (m_pDVDAuthor->getSubMenuByName (qsMenuName))
			bEditable = false;
	}
	m_pComboMenu->setEditable ( bEditable );
}

void TemplateWizard::slotTemplateClicked (QMouseEvent *pMouseEvent)
{
	uint t;
	QPoint mousePos;
	QString qsFileName, qsFilter, qsExtensions;
	DropZone *pDropZone = NULL;
	
	mousePos.setX ((int)( pMouseEvent->pos ().x() / m_fScale ));
	mousePos.setY ((int)( pMouseEvent->pos ().y() / m_fScale ));
	
	for (t=0;t<m_listDropZones.count();t++)	{
		if (m_listDropZones[t]->rect.contains (mousePos))	{
			pDropZone = m_listDropZones[t];
			break;
		}
	}
	if ( ! pDropZone )
		return;

	qsFilter   = m_pDVDAuthor->getImageFilter ();
	qsFilter  += m_pDVDAuthor->getMovieFilter ();
	qsFileName = QFileDialog::getOpenFileName ( Global::qsCurrentPath, QString ("Images/Movies (") + qsFilter + QString (")"));
	if (qsFileName.isNull())
		return;

	QImage theImage;
	Utils  theUtil;

	QFileInfo backgroundInfo (qsFileName);
	MediaInfo *pMediaInfo = m_pDVDAuthor->getMediaInfo ();

	Global::qsCurrentPath = backgroundInfo.absFilePath();

	pDropZone->qsFileName = qsFileName;
	qsExtensions = pMediaInfo->getExtensions (true);
	if (qsExtensions.find ("."+backgroundInfo.extension(FALSE), 0, FALSE) > -1)	{
		// Okay let us get the first frame of the game as a screenshot ...
		pMediaInfo->setFileName(qsFileName);
		theImage = pMediaInfo->getScreenshot(0);
		// The we save the screenshot in the temp dir
		qsFileName = theUtil.getTempFile(backgroundInfo.baseName())+QString (".png");
		theImage.save (qsFileName, "PNG");
		pDropZone->bMovie = true;
	}
	else	// Not a movie file, then we should reset this ...
		theImage = QImage(qsFileName);

	theImage = theImage.smoothScale ((int)(m_fScale*pDropZone->rect.width()), (int)(m_fScale * pDropZone->rect.height()), QImage::ScaleMin);

	if (pDropZone->pPixmap)
		delete pDropZone->pPixmap;
	pDropZone->pPixmap = new QPixmap;
	pDropZone->pPixmap->convertFromImage (theImage);
	refreshBackground ();
}

QString TemplateWizard::readProjectFile (QString qsFileName, bool bDescriptionOnly)
{
	QDomDocument xmlDoc( TEMPLATE_DOCTYPE );
	QDomElement  docElem;
	QDomNode     xmlNode;
	int          iX, iY, iWidth, iHeight, iDropZoneNumber;
	uint         t;
	QString      text;
	QRect        rectDropZone;
	// First we take care of the QDVDAuthor variables ...
	QString qsVersionNumber, qsTemplateName, qsReturn;
	QDomAttr attribute;

	QFile templateFile (qsFileName);
	if ( ! templateFile.open(IO_ReadWrite))
		return qsReturn;
	
	if (!xmlDoc.setContent (&templateFile))	{
		// Error handling ...
		templateFile.close();
		QMessageBox::warning ( this, tr ("xml template file seems to be defective."),
			tr ("Please try another template file."),
			QMessageBox::Ok, QMessageBox::NoButton);
		return qsReturn;
	}

	for (t=0;t<m_listDropZones.count ();t++)
		delete m_listDropZones[t];
	m_listDropZones.clear ();

	// Here is the main loop to extract the info ...
	docElem = xmlDoc.documentElement();
	xmlNode = docElem.firstChild();

	attribute = docElem.attributeNode ( TEMPLATE_NAME );
	if (!attribute.isNull())	{
		qsTemplateName = attribute.value();
		m_qsCurrentTemplate = qsTemplateName;
	}
//printf ("TemplateWizard::getDescription <%s>\n", (const char *)attribute.value());
	while( !xmlNode.isNull() ) {
		QDomElement searchXMLTree = xmlNode.toElement();
		// Okay, here we retain the stored data from the xml file.
		QString tagName  = searchXMLTree.tagName();
		QString nodeText = searchXMLTree.text ();
		// Okay, this is ugly but after all it is the simplest of all xml file structure.
		// No need to get fancy ...
		if (QDVDAUTHOR_VERSION == tagName)
			qsVersionNumber = nodeText;
		else if (TEMPLATE_DESCRIPTION == tagName)
			qsReturn = nodeText;
		else if ( (TEMPLATE_DROPZONE == tagName ) && (!bDescriptionOnly) )	{
			attribute = searchXMLTree.attributeNode ( TEMPLATE_DROPZONE_NUMBER);
			iDropZoneNumber = attribute.value().toInt();
			attribute = searchXMLTree.attributeNode ( TEMPLATE_DROPZONE_X);
			iX = attribute.value().toInt();
			attribute = searchXMLTree.attributeNode ( TEMPLATE_DROPZONE_Y);
			iY = attribute.value().toInt();
			attribute = searchXMLTree.attributeNode ( TEMPLATE_DROPZONE_WIDTH);
			iWidth = attribute.value().toInt();
			attribute = searchXMLTree.attributeNode ( TEMPLATE_DROPZONE_HEIGHT);
			iHeight = attribute.value().toInt();
			rectDropZone = QRect(iX, iY, iWidth, iHeight);
			DropZone *pDropZone = new DropZone (iDropZoneNumber, text, rectDropZone);
			m_listDropZones.append (pDropZone);
		}
		else if ( (TEMPLATE_DATA == tagName ) && (!bDescriptionOnly) )	{
			m_qsXMLDVDMenu = searchXMLTree.text();
		}
		// So lets get the next sibling ... until we hit hte end of DVDMenu ...
		xmlNode = xmlNode.nextSibling();
	}
	templateFile.close();

	// Okay, once something has been selected means we keep this and enable the next button
	nextButton()->setEnabled (true);
	
	return qsReturn;
}

void TemplateWizard::next ()
{
	QString qsFileName;
	// The user clicked the next button.
	if (m_qsPathName.isEmpty ())
		return;
	if ( ! m_pComboMenu->editable() )	{
		DVDMenu *pDVDMenu = m_pDVDAuthor->getSubMenuByName(m_pComboMenu->currentText());
		if ( ( pDVDMenu) && ( ! pDVDMenu->isEmpty () ) )	{
			if (QMessageBox::information (NULL, tr ("Override Menu"), tr ("Warning the menu %1 will be replaced.\nAre you sure ?").arg (m_pComboMenu->currentText()), QMessageBox::Yes,  QMessageBox::No) == QMessageBox::No)
			return;
		}
	}

	QWizard::next ();
	// At this point we should be on the other side ...
	// So first we read in the whole project file.
	qsFileName = m_qsPathName + QString ("template.xml");
	readProjectFile (qsFileName, false);
	// Then we build a simple GUI for the user to provide the neccesary files
	refreshBackground ();
}

void TemplateWizard::refreshBackground ()
{
	uint t;
	int iTemp, iX, iY;
	QPixmap thePixmap;
	QImage theImage;
	QString qsFileName;
	QRect tempRect;

	theImage = QImage (m_qsPathName + QString ("preview.png"));
	theImage = theImage.smoothScale (m_pFrameProperties->width(), m_pFrameProperties->height(), QImage::ScaleMin);
	thePixmap.convertFromImage(theImage);
	// Next we draw the black DropZones ...
	m_fScale = (float)thePixmap.width() / 720.0;
	QPainter painter( &thePixmap );
	painter.setBrush ( QColor (0, 0, 0) );
	for (t=0;t<m_listDropZones.count();t++)	{
		if (m_listDropZones[t]->pPixmap == NULL)	{
			tempRect.setX     ((int) (m_fScale * m_listDropZones[t]->rect.x()));
			tempRect.setY     ((int) (m_fScale * m_listDropZones[t]->rect.y()));
			tempRect.setWidth ((int) (m_fScale * (m_listDropZones[t]->rect.width ()+2)));
			tempRect.setHeight((int) (m_fScale * (m_listDropZones[t]->rect.height()+2)));
			painter.drawRect (tempRect);
		}
		else	{
			iTemp = (int)(m_fScale * m_listDropZones[t]->rect.width()) - m_listDropZones[t]->pPixmap->width();
			iX = (int)(m_fScale * m_listDropZones[t]->rect.x()+(int)(iTemp/2.0));
			iTemp = (int)(m_fScale * m_listDropZones[t]->rect.height()) - m_listDropZones[t]->pPixmap->height();
			iY = (int)(m_fScale * m_listDropZones[t]->rect.y()+(int)(iTemp/2.0));
			painter.drawPixmap(iX, iY, *(m_listDropZones[t]->pPixmap));
		}
	}
	m_pFrameProperties->setFixedWidth  (thePixmap.width ());
	m_pFrameProperties->setFixedHeight (thePixmap.height());
	m_pFrameProperties->setBackgroundPixmap (thePixmap);
}

DVDMenu *TemplateWizard::getMenu ()
{
	return m_pDVDMenu;
}

void TemplateWizard::reject ()
{
	if (m_pDVDMenu)
		delete m_pDVDMenu;
	m_pDVDMenu = NULL;
	QWizard::reject();
}

void TemplateWizard::accept ()
{
	// First we are going to replace the DropZone placeholder and then create a DVDMenu.
	uint t;
	int i, iFreeSlot, iX, iY, iWidth, iHeight;
	QString qsAction, qsEntryName, qsPrefixDir, qsMenuName, qsGeometry, qsCurrentPath;
	QFileInfo fileInfo;
	QStringList listAddMovies;
 
	QCursor myCursor (QCursor::WaitCursor);
	setCursor(myCursor);
	qsCurrentPath = Global::qsCurrentPath;
	iFreeSlot = m_pDVDAuthor->getFreeSourceSlot ();
	qsEntryName.sprintf ("[%02d] - basic6", iFreeSlot);
	
	qsPrefixDir = QString ("%1/share/qdvdauthor/").arg( PREFIX_DIRECTORY );
	m_qsXMLDVDMenu.replace (QString ("+PREFIX_DIRECTORY+"), qsPrefixDir);
	m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_SOURCE_ENTRY+"), qsEntryName);
	for (t=0;t<m_listDropZones.count();t++)	{
		i = m_listDropZones[t]->iNumber;
		if (m_listDropZones[t]->qsFileName.isEmpty())	{
			m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_OBJECT_TYPE+").arg(i), QString ("IgnoreObject"));
			m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_BUTTON+").arg(i), QString ("IgnoreObject"));
			m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_FILE_NAME+").arg(i), QString ());
			m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_BASE_NAME+").arg(i), QString ());
			m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_GEOMETRY+").arg(i), QString ());
			m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_MOVIE_SIZE+").arg(i), QString ());
			m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_ACTION+").arg(i), QString ());
			continue;
		}
		fileInfo.setFile(m_listDropZones[t]->qsFileName);
		// Next we take care of the geometry
		iWidth  = (int) (m_listDropZones[t]->pPixmap->width() /m_fScale+0.5);
		iHeight = (int) (m_listDropZones[t]->pPixmap->height()/m_fScale+0.5);
		if (iWidth<2)
			iWidth = m_listDropZones[t]->rect.width ();
		if (iHeight<2)
			iHeight= m_listDropZones[t]->rect.height();
		iX      = (int)((m_listDropZones[t]->rect.width () - iWidth )/2.0 + m_listDropZones[t]->rect.x());
		iY      = (int)((m_listDropZones[t]->rect.height() - iHeight)/2.0 + m_listDropZones[t]->rect.y());
		qsGeometry = QString (" X=\"%1\" Y=\"%2\" Width=\"%3\" Height=\"%4\" ").arg(iX).arg(iY).arg(iWidth).arg(iHeight);

		m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_FILE_NAME+").arg(i), fileInfo.absFilePath());
		m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_BASE_NAME+").arg(i), fileInfo.baseName());
//		m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_BASE_NAME+").arg(i), fileInfo.fileName());
		m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_BUTTON+").arg(i), QString ("ButtonObject"));
		m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_GEOMETRY+").arg(i), qsGeometry);
		if (m_listDropZones[t]->bMovie)	{
			qsAction = QString ("jump+-+%1+-+%2+-+00:00:00.000").arg(qsEntryName).arg(fileInfo.fileName());
			m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_OBJECT_TYPE+").arg(i), QString ("MovieObject"));
			m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_MOVIE_SIZE+").arg(i), QString ("720x480"));
			listAddMovies.append (m_listDropZones[t]->qsFileName);
		}
		else	{	// The image button should jump to the MainMenu  jump+-+vmgm+-+
			qsAction = QString ("jump+-+vmgm+-+");
			m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_OBJECT_TYPE+").arg(i), QString ("ImageObject"));
		}
		m_qsXMLDVDMenu.replace (QString ("+DROP_ZONE_%1_ACTION+").arg(i), qsAction);
	}

	if (listAddMovies.count())
		m_pDVDAuthor->addMovieList (listAddMovies, &qsEntryName);

	QDomDocument domDoc;
	int iRow, iCol;
	QString qsError;

	iRow = iCol = 0;
	domDoc.setContent (m_qsXMLDVDMenu, true, &qsError, &iRow, &iCol);
//	printf ("\n\n%s\n\n", (const char *)m_qsXMLDVDMenu);
	QDomElement  menuElement = domDoc.documentElement();
	if (m_pDVDMenu)
		delete m_pDVDMenu;

	// if the menu exist we take this one over
	m_pDVDMenu = m_pDVDAuthor->getSubMenuByName(m_pComboMenu->currentText());
	if ( m_pDVDMenu )	{
		qsMenuName = m_pComboMenu->currentText();
		m_pDVDMenu->clear ();
		m_pDVDMenu->readProjectFile(menuElement);
		m_pDVDMenu->getInterface()->qsMenuName = qsMenuName;
		m_pDVDMenu->setTabLabel (qsMenuName);
		m_pDVDMenu->updateDVDMenu();
	}
	else	{ // Otherwise we will create a new menu.
		m_pDVDMenu = new DVDMenu(m_pDVDAuthor);
		qsMenuName = m_pDVDAuthor->getUniqueMenuTitle (m_pComboMenu->currentText(), 0);
		m_pDVDMenu->readProjectFile(menuElement);
		m_pDVDMenu->getInterface()->qsMenuName = qsMenuName;
		m_pDVDMenu->setTabLabel (qsMenuName);
		m_pDVDMenu->updateDVDMenu();
		m_pDVDAuthor->addDVDMenu(m_pDVDMenu);
	}
	Global::qsCurrentPath = qsCurrentPath;

	m_pDVDAuthor->slotAutosave ();
	m_pDVDAuthor->slotUpdateStructure ();
	// Here we have to create the SoourceFileEntry for the DVDAuthorObject
	// Note, the SourceFileEntry holds N SourceFileInfo's
	QWizard::accept();
	QTimer::singleShot (10, m_pDVDAuthor, SLOT(slotDeleteTemplateWizard()));
	myCursor = QCursor(QCursor::ArrowCursor);
	setCursor(myCursor);
}


